/**************************************
 *   C _ U R A V         2 . 0        *
 *        *
 *               *
 *    -                  *
 *                                    * 
 *                   *
 *                                    *
 *   (C) 2002-2005       *
 *                 *
 **************************************/
/*       :
	    
	[Cu(NH3)4]SO4*H2O=CuO+SO3+NH3+H2O
	CaSO4*0.5H2O+H2O=CaSO4*2H2O
	Cr2O7{2-}+H{+}+e=Cr{3+}+H2O
	Fe{3+}+e=Fe  ..
	        -
	(, H2O  ),   .

	:
	1.      
	      ()  []     *
	2.         
	    2NO2 + O3 = N2O5 + O2
	3.      20 
	   / 
	4.    +  -  :
	   Sn{++} SO4{--}  . . */

/***   ***/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "gauss2.inc"

/**********  **********/
#define ACCURACY GAUSS_ACCURACY
#define ERR_MANYSOL "     "
#define ERR_NOSOL "   "
#define ERR_EQU "  '='"
#define ERR_PLUS "  '+'"
#define ERR_SYNT " "
#define ERR_WRONGEL " %c%c    "
#define ERR_CHARGE "   "
#define ERR_MANYELEM "  "
#define ERR_MANYCOMP "  "
#define ERR_MANYBR "  "
#define ERR_SYNTBR "  "

/**********   **********/
double rou(double x);
char cmpzero(double x);

/**********  **********/
struct composition /*      */
{
	char name[64];   /*   */
	int charge;      /*   */
};

struct brackets /*      */
{
	unsigned char open_sign; /*    */
	unsigned char close_sign; /*    */
	int open_pos;        /*    */
	int close_pos;       /*    */
	double index;        /*    */
};

/**********   **********/
/*     
     .   
       .

    char *in  -  
    char *res -  

      0   
                      1    

         
    ,    
      */

/***    c_urav ***/
/*     */
#define PRINT(fmt, arg) {sprintf(buf, fmt, arg); \
                         strcat(res, buf);}

#define LPRINT(arg) {sprintf(buf, arg);\
                     strcat(res, buf);}

/*      
       */
#define errmessage(txt) {sprintf(res,"%s",txt); return 1;}

int c_urav(char *in, char *res)
{	char buf[100];               /*    */
	char eq[256];                /*    */
	struct composition comp[20]; /*   */
	char elem[60] = {0};	     /*     .*/
				     /* 2     */
	char ce[3]={0};	             /*   .  */
	double a[20][20] = {0};      /*     -*/
	double x[20];                /*    . */
	int ncomp; /* (   ) - 1 */
	int nelem; /* (   ) - 1 */

	struct brackets br[4] =      /*      */
	{'[',']',-1,-1,1,
	 '(',')',-1,-1,1,
	 '*',13,-1,-1,1,
	 '{','}',-1,-1,1};

	double ta, tb;          /*   */
	char *cptr;
	int i, j, k;            /*   */
	unsigned char c, c1;    /*   */
	double ii;              /*    */
	int n;                  /*    . */

	/***** I.    *****/
	res[0] = 0;
	for (i = 0, j = 0; i < strlen(in); i++)
		if (in[i] != ' ') eq[j++] = in[i];
	eq[j] = 0;	

	/***** II.     ***/
	i = -1; /*    */
	j = 0;  /*     */
	k = 0;  /*    */
	n = 1;  /*  
		  0  
		  1   */
	ncomp = 0; /*    */
	do
	{
		c = eq[++i];
		if (c == '{') n = 0;
		if (c == '}') n = 1;
		if ( (c == '+' && n) || c == '=')
		{
			if (c == '=') k++;
			comp[ncomp++].name[j] = 0;
			j = 0;
		}
		else
			comp[ncomp].name[j++] = c;
	}
	while (c);
	if (!n) errmessage(ERR_SYNT);

	/***** III.    *****/
	if (ncomp > 19) errmessage(ERR_MANYCOMP);
	if (k != 1) /*   1  '=' */
		errmessage(ERR_EQU);

	for (i = 0; i <= ncomp; i++)
	{
		c = comp[i].name[0];
		if (c == 0) /*    2 '+'  */
			errmessage(ERR_PLUS);
		if ( !(c >= 'A' && c <= 'Z') && c != 'e' && c != '(' && c != ')' && c != '[' && c != ']')
			errmessage(ERR_SYNT);
	}

	/***** IV.    -   ***/
	nelem = -1;
	for (i = 0; i <= ncomp; i++)
	{
		/***      ***/
		for (j = 0; j <= 3; j++) /*    .  */
		{	br[j].open_pos = -1;
			br[j].close_pos = -1;}

		for (j = 0; j <= strlen(comp[i].name); j++) /*   */
		{
			c = comp[i].name[j];
			for (k = 0; k <= 3; k++)
			{
				if (c == br[k].open_sign) /*   */
				{ if (br[k].open_pos != -1) errmessage(ERR_MANYBR);
				  br[k].open_pos = j;
				  if (k == 2) /*  */
				  {	  br[2].close_pos = strlen(comp[i].name) + 1;
					  br[2].index = strtod(&comp[i].name[j + 1],&cptr);
					  if (br[k].index == 0) br[k].index = 1;}
				}

				if (c == br[k].close_sign) /*   */
				{       if (br[k].close_pos != -1) errmessage(ERR_MANYBR);
					br[k].close_pos = j;
					br[k].index = strtod(&comp[i].name[j + 1],&cptr);
					if (br[k].index == 0) br[k].index = 1;}
			}
		}
		/*    */
		for (k = 0; k <= 3; k++)
			if ((br[k].open_pos == -1 && br[k].close_pos != -1)||(br[k].open_pos > br[k].close_pos)) errmessage(ERR_SYNTBR);

		/*    */
		if (br[3].open_pos != -1 && br[3].close_pos != -1)
		{	/*    */
			comp[i].charge = strtol(&comp[i].name[br[3].open_pos + 1], &cptr, 10);
			/*   */
			c = comp[i].name[br[3].close_pos - 1];
			if ((c == '-' || c == '+') && comp[i].charge == 0) comp[i].charge = 1;
			if (c == '-') comp[i].charge = -comp[i].charge;
		}
		else comp[i].charge = 0;

		/*     */
		if (comp[i].name[0] == 'e')
		{	comp[i].charge = -1;
			continue;}

		/*     */
		for (j = 0; j <= strlen(comp[i].name); j++)
		{
			/*    */
			c = comp[i].name[j];
			if (c < 'A' || c > 'Z') continue;
			ce[0] = c;

			/*      */
			c1 = comp[i].name[j + 1];
			ce[1] = (c1 < 'a' || c1 > 'z') ? 32 : c1;

			/*   */
			if (ce[1] == 32)
				ii = strtod(&comp[i].name[j + 1], &cptr);
			else
				ii = strtod(&comp[i].name[j + 2], &cptr);

			if (ii == 0) ii = 1; /*     */
					     /*  - 1*/

			/*     */
			for (k = 0; k <= 2; k++)
				if (j > br[k].open_pos && j < br[k].close_pos) ii *= br[k].index;

			/*      */
			if (strstr(elem, ce) == NULL)
			{	strcat(elem, ce);
				nelem++; }

			/*         */
			n = (strstr(elem, ce) - &elem[0]) / 2;
			a[n][i] += ii;
		}
	}
	if (nelem > 19) errmessage(ERR_MANYELEM); /*     ? */

	/***** V.         *****/
	for (i = 0; i <= nelem; i++)
	{
		n = 0;
		for (j = 0; j <= ncomp; j++) if (a[i][j]) n++;
		if (n < 2)
		{
			sprintf(res, ERR_WRONGEL, elem[i * 2], elem[i * 2 + 1]);
			return 1;
		}
	}

	/***** VI.    
		          ***/
	n = 0;
	for (i = 0; i <= ncomp; i++)
	{
		a[nelem + 1][i] = comp[i].charge;
		if (comp[i].charge) n++;
	}
	if (n == 1) errmessage (ERR_CHARGE);/*   1 .  */
	if (n) nelem++;                     /*    */

	/***** VII.    *****/
	i = gauss(a, ncomp, nelem + 1, x);
	if (i == GAUSS_NOSOL) errmessage(ERR_NOSOL);
	if (i == GAUSS_MANYSOL) errmessage(ERR_MANYSOL);
	x[ncomp] = 1;

	/***** VIII.      *****/
	for (i = 1; i <= 1000; i++)
	{
		/*   i   */
		k = 1;
		for (j = 0; j <= ncomp; j++)
		{
			ta = x[j] * (double)i;
			tb = rou(ta) - ta;
			if (cmpzero(tb)) {k = 0; break;}
		}

		/*   */
		if (k == 1)
		{
			for (j = 0; j <= ncomp; j++) x[j] *= (double)i;
			break;
		}
	}

   /***** IX.   *****/
   /*   .  - 
        -  */
   if (x[0] < 0) k = -1; else k = 1;
   for (i = 0; i <= ncomp; i++) x[i] *= k;

   /* )    */
   j = 0; /*  0 - -   */
   for (i = 0; i <= ncomp; i++)
   {
    /*   i   */
    if (x[i] <= 0) continue;
    if (i > 0 && j) PRINT("%s", " + "); /*   -  */
    j++;
    if (cmpzero(x[i] - 1.0)) PRINT("%.0f ",x[i]);
    PRINT("%s",comp[i].name);
   }
   PRINT("%s", " = "); /*    */

   /* )   */
   j = 0; /*  0 - -   */
   for (i = 0; i <= ncomp; i++)
   {
    /*   i   */
    if (x[i] >= 0) continue;
    if (i > 0 && j) LPRINT(" + "); /*   -  */
    j++;
    if (cmpzero(-x[i] - 1.0)) PRINT("%.0f ",-x[i]);
    PRINT("%s",comp[i].name);
   }
   return 0;
}

/**********   **********/
/*      :
   3.6 = 4  -3.6 = - 4   3.1 = 3   -3.1 = 3 */
double rou(double x)
{
 double c, d;
 d = modf(x, &c);
 if (fabs(d) < 0.5) return c;
 if (x < 0) return c - 1.0;
 if (x > 0) return c + 1.0;
}

/*       ACCURACY */
/* 0 -  0
   1 -   0 */
char cmpzero(double x)
{
	return (fabs(x) > ACCURACY);
}
